local Castbars = LibStub("AceAddon-3.0"):NewAddon("Castbars");

Castbars.SharedMedia = LibStub("LibSharedMedia-3.0");

function Castbars:FrameMediaRestore(frame)
    local barTexture = self.SharedMedia:Fetch('statusbar', self.db.profile[frame.configName]["Texture"]);
    local borderTexture = self.SharedMedia:Fetch('border', self.db.profile[frame.configName]["Border"]);
    local font = self.SharedMedia:Fetch('font', self.db.profile[frame.configName]["Font"]);
    if (barTexture) then
        frame.statusBar:SetStatusBarTexture(barTexture);
    end
    if (borderTexture) then
        frame.backdrop:SetBackdrop({edgeFile = borderTexture, edgeSize = 16, tileSize = 16, insets = {bottom = 5, top = 5, right = 5, left = 5}});
        local brightness = self.db.profile[frame.configName]["BorderBrightness"];
        frame.backdrop:SetBackdropBorderColor(brightness, brightness, brightness, 1.0);
    end
    if (font) then
        local textSize = self.db.profile[frame.configName]["FontSize"];
        local outline = self.db.profile[frame.configName]["FontOutline"] and "OUTLINE" or nil;
        frame.text:SetFont(font, textSize, outline);
        if (frame.timer) then
            frame.timer:SetFont(font, textSize, outline);
        end
    end
end

function Castbars:FrameMediaRestoreAll()
    for i, frame in pairs(self.frames) do
        self:FrameMediaRestore(frame)
    end
end

function Castbars:FrameRestoreOnCast(frame)
    if (frame.shield and frame.shield:IsShown()) then
        frame.statusBar:SetStatusBarColor(0.7, 0.7, 0.7);
    end
    if (frame.icon) then
        if (frame.shield and frame.shield:IsShown() and (self.db.profile[frame.configName]["ShowShield"] == true)) then
            local hb, wb = frame.shield:GetHeight(), frame.shield:GetWidth();
            frame.icon:ClearAllPoints();
            frame.icon:SetPoint("TOPRIGHT", frame, "TOPLEFT", -0.03515625 * wb, 0.09375 * hb);
            frame.icon:SetHeight(0.328125 * hb);
            frame.icon:SetWidth(0.08203125 * wb);
            frame.icon:Show();
        elseif (self.db.profile[frame.configName]["ShowIcon"]) then
            frame.icon:ClearAllPoints();
            if (self.db.profile[frame.configName]["Border"] == "None") then
                frame.icon:SetPoint("RIGHT", frame, "LEFT", 0, 0);
                frame.icon:SetHeight(self.db.profile[frame.configName]["Height"]);
                frame.icon:SetWidth(self.db.profile[frame.configName]["Height"]);
            else
                frame.icon:SetPoint("RIGHT", frame, "LEFT", -5, 0);
                frame.icon:SetHeight(self.db.profile[frame.configName]["Height"] + 5);
                frame.icon:SetWidth(self.db.profile[frame.configName]["Height"] + 5);
            end
            frame.icon:Show();
        else
            frame.icon:Hide();
        end
    end
end

function Castbars:FrameLayoutRestore(frame)
    -- Position
    local position = self.db.profile[frame.configName]["Position"];
    if (frame.dragable and position) then
        frame:ClearAllPoints();
        frame:SetPoint(position.point, position.parent, position.relpoint, position.x, position.y);
    end

    -- Texture
    self:FrameMediaRestore(frame);

    -- Dimensions
    frame:SetWidth(self.db.profile[frame.configName]["Width"]);
    frame:SetHeight(self.db.profile[frame.configName]["Height"]);

    -- Restore dynamic settings
    self:FrameRestoreOnCast(frame);

    -- Visibility
    if (self.db.profile[frame.configName]["Show"] == false) then
        frame.showCastbar = false;
    else
        frame.showCastbar = true;
    end

    -- Shield visibility
    if (frame.shield) then
        if (self.db.profile[frame.configName]["ShowShield"] == false) then
            frame.shield:SetTexture();
        else
            frame.shield:SetTexture("Interface\\CastingBar\\UI-CastingBar-Small-Shield");
            frame.shield:SetGradientAlpha("HORIZONTAL", 1, 1, 1, 0.5, 1, 1, 1, 0.5);
        end
    end

    -- Text Alignment
    frame.text:SetJustifyH(self.db.profile[frame.configName]["TextAlignment"]);
end

function Castbars:FrameLayoutRestoreAll()
    for i, frame in pairs(self.frames) do
        self:FrameLayoutRestore(frame)
    end
end

function Castbars:FrameCustomize(frame)
    local frameName = frame:GetName();
    local frameType;

    if (frameName:sub(1, 11) == "MirrorTimer") then
        frameType = "Mirror";
        frame.statusBar = _G[frameName .. "StatusBar"];
        frame.configName = "MirrorTimer";
        if (tonumber(frameName:sub(12,12)) == 1) then
            frame.dragable = true;
        end
    elseif (frameName == "CastingBarFrame" or frameName == "TargetCastingBarFrame" or frameName == "PetCastingBarFrame") then
        frameType = "Castbar";
        frame.statusBar = frame;
        if (frameName == "PetCastingBarFrame") then
            frame.configName = self.db.profile.PetCastingBarFrame["Merge"] and "CastingBarFrame" or frameName;
        else
            frame.configName = frameName;
        end
        frame.dragable = true;
        frame.icon = _G[frameName .. "Icon"];
        frame.shield = _G[frameName .. "BorderShield"];
    end

    -- Make dragable
    if (frame.dragable) then
        frame:SetMovable(true);
        frame:RegisterForDrag("LeftButton");
        frame:SetScript("OnDragStart", function(frame)
            if (self.ConfigMode) then
                frame:StartMoving();
            end
        end);
        frame:SetScript("OnDragStop", function(frame)
            frame:StopMovingOrSizing();
            if (not self.db.profile[frame.configName]["Position"]) then
                self.db.profile[frame.configName]["Position"] = {};
            end
            local position = self.db.profile[frame.configName]["Position"];
            position.point, position.parent, position.relpoint, position.x, position.y = frame:GetPoint();
        end);
    end

    -- Adjust spark position
    frame.spark = _G[frameName .. "Spark"];
    if (frame.spark) then
        local setPoint = frame.spark.SetPoint;
        frame.spark.SetPoint = function(self, point, relativeFrame, realativePoint, x, y)
            setPoint(self, point, relativeFrame, realativePoint, x, 0);
        end
    end

    -- Adjust text position
    frame.text = _G[frameName .. "Text"];
    frame.text:ClearAllPoints();
    if (frameType == "Castbar") then
        frame.text:SetPoint("LEFT", frame.statusBar, "LEFT", 5, 1);
    elseif (frameType == "Mirror") then
        frame.text:SetPoint("CENTER", frame.statusBar, "CENTER", 0, 1);
    end

    -- Remove the border
    local frameBorder = _G[frameName .. "Border"];
    frameBorder:SetTexture();

    -- Remove the flash
    local frameFlash = _G[frameName .. "Flash"];
    if (frameFlash) then
        frameFlash:SetTexture();
    end

    -- Create backdrop
    frame.backdrop = CreateFrame("Frame", frameName .. "Backdrop", frame);
    if (frameType == "Castbar") then
        frame.backdrop:SetPoint("CENTER", frame, "CENTER", 0, 0);
    elseif (frameType == "Mirror") then
        frame.backdrop:SetPoint("CENTER", frame, "CENTER", 0, 3);
    end

    if (frameType == "Castbar") then
        -- Add timer text
        frame.timer = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlight");
        frame.timer:SetPoint("RIGHT", frame, "RIGHT", -5, 1);
        frame.nextupdate = 0.1;

        -- Add latency indication texture
        frame.latency = frame:CreateTexture(nil, "ARTWORK");
        frame.latency:Hide();
        frame.latency:SetHeight(frame:GetHeight());
        frame.latency:SetPoint("RIGHT", frame, "RIGHT", 0, 0);
        frame.latency:SetTexture(1, 0, 0, 0.65);

        -- Adjust shield layer
        frame.shield:SetDrawLayer("OVERLAY");
    elseif (frameType == "Mirror") then
        for i, region in pairs({frame:GetRegions()}) do
            if (region.GetTexture and region:GetTexture() == "SolidTexture") then
                frame.shade = region;
            end
        end
    end

    -- Automatically adjust the height of sub elements
    local setHeight = frame.SetHeight;
    if (frameType == "Castbar") then
        frame.SetHeight = function(self, height)
            frame.backdrop:SetHeight(height + 10);
            frame.spark:SetHeight(2.5 * height);
            frame.latency:SetHeight(height);
            frame.shield:SetHeight(5.82 * height);
            setHeight(self, height);
        end
    elseif (frameType == "Mirror") then
        frame.SetHeight = function(self, height)
            frame.backdrop:SetHeight(height + 10);
            frame.statusBar:SetHeight(height);
            frame.shade:SetHeight(height);
            setHeight(self, height + 10);
        end
    end

    -- Automatically adjust the width of sub elements
    local setWidth = frame.SetWidth
    if (frameType == "Castbar") then
        frame.SetWidth = function(self, width)
            frame.backdrop:SetWidth(width + 10);
            frame.timer:SetText("0.0");
            frame.text:SetWidth(width - 10 - frame.timer:GetWidth());
            frame.shield:SetWidth(1.36 * width);
            frame.shield:ClearAllPoints();
            frame.shield:SetPoint("CENTER", frame.statusBar, "CENTER", -0.031875 * width, 0);
            setWidth(self, width);
        end
    elseif (frameType == "Mirror") then
        frame.SetWidth = function(self, width)
            frame.backdrop:SetWidth(width + 10);
            frame.statusBar:SetWidth(width);
            frame.shade:SetWidth(width);
            setWidth(self, width + 10);
        end
    end

    frame:Hide();
end

function Castbars:FrameCustomizeAll()
    for i, frame in pairs(self.frames) do
        self:FrameCustomize(frame)
    end
end

function Castbars:GetOptionsTableForBar(frameConfigName, friendlyName, order, textAlign, showHideShield, enableDisable, showHideIcon, disabled)
    local options =
    {
        type = "group",
        name = friendlyName .. " Options",
        guiInline = true,
        order = order,
        hidden = disabled,
        disabled = disabled,
        args =
        {
            width =
            {
                name = "Width",
                type = "range",
                order = 10,
                desc = "Set the width of the " .. friendlyName,
                min = 100,
                max = 300,
                step = 1,
                get = function() return self.db.profile[frameConfigName]["Width"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["Width"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            height =
            {
                name = "Height",
                type = "range",
                order = 11,
                desc = "Set the height of the " .. friendlyName,
                min = 10,
                max = 50,
                step = 1,
                get = function() return self.db.profile[frameConfigName]["Height"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["Height"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            texture =
            {
                name = "Texture",
                type = 'select',
                order = 12,
                desc = "Select texture to use for the " .. friendlyName,
                dialogControl = 'LSM30_Statusbar',
                values = AceGUIWidgetLSMlists.statusbar,
                get = function() return self.db.profile[frameConfigName]["Texture"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["Texture"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            font =
            {
                name = "Font",
                type = 'select',
                order = 13,
                desc = "Select font to use for the " .. friendlyName,
                dialogControl = 'LSM30_Font',
                values = AceGUIWidgetLSMlists.font,
                get = function() return self.db.profile[frameConfigName]["Font"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["Font"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            fontsize =
            {
                name = "Font Size",
                type = "range",
                order = 14,
                desc = "Set the font size of the " .. friendlyName,
                min = 6,
                max = 30,
                step = 1,
                get = function() return self.db.profile[frameConfigName]["FontSize"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["FontSize"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            fontoutline =
            {
                name = "Font Outline",
                type = "toggle",
                order = 15,
                desc = "Toggles outline on the font of the " .. friendlyName,
                get = function() return self.db.profile[frameConfigName]["FontOutline"] end,
                set = function()
                    self.db.profile[frameConfigName]["FontOutline"] = not self.db.profile[frameConfigName]["FontOutline"];
                    self:FrameLayoutRestoreAll();
                end,
            },
            border =
            {
                name = "Border",
                type = 'select',
                order = 20,
                desc = "Select border to use for the " .. friendlyName,
                dialogControl = 'LSM30_Border',
                values = AceGUIWidgetLSMlists.border,
                get = function() return self.db.profile[frameConfigName]["Border"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["Border"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
            borderbrightness =
            {
                name = "Border Brightness",
                type = "range",
                order = 21,
                desc = "Set the brightness of the border of the " .. friendlyName,
                min = 0.0,
                max = 1.0,
                step = 0.05,
                isPercent = true,
                get = function() return self.db.profile[frameConfigName]["BorderBrightness"] end,
                set = function(info, value)
                    self.db.profile[frameConfigName]["BorderBrightness"] = value;
                    self:FrameLayoutRestoreAll();
                end,
            },
        },
    };
    if (enableDisable) then
        options.args.enable =
        {
            name = "Enable",
            type = "toggle",
            order = 1,
            desc = "Toggles display of the " .. friendlyName,
            get = function() return self.db.profile[frameConfigName]["Show"] end,
            set = function()
                self.db.profile[frameConfigName]["Show"] = not self.db.profile[frameConfigName]["Show"];
                self:FrameLayoutRestoreAll();
            end,
        }
    end
    if (showHideIcon) then
        options.args.showicon =
        {
            name = "Show Icon",
            type = "toggle",
            order = 2,
            desc = "Toggles display of the icon at the left side of the bar",
            get = function() return self.db.profile[frameConfigName]["ShowIcon"] end,
            set = function()
                self.db.profile[frameConfigName]["ShowIcon"] = not self.db.profile[frameConfigName]["ShowIcon"];
                self:FrameLayoutRestoreAll();
            end,
        }
    end
    if (showHideShield) then
        options.args.showshield =
        {
            name = "Show Shield",
            type = "toggle",
            order = 3,
            desc = "Toggles display of the shield around the bar when the spell cannot be interrupted.",
            get = function() return self.db.profile[frameConfigName]["ShowShield"] end,
            set = function()
                self.db.profile[frameConfigName]["ShowShield"] = not self.db.profile[frameConfigName]["ShowShield"];
                self:FrameLayoutRestoreAll();
            end,
        }
    end
    if (textAlign) then
        options.args.textalign =
        {
            name = "Text Alignment",
            type = "select",
            order = 16,
            desc = "Set the alignment of the Castbar text",
            values = {LEFT = "Left", CENTER = "Center"},
            get = function() return self.db.profile[frameConfigName]["TextAlignment"] end,
            set = function(info, value)
                self.db.profile[frameConfigName]["TextAlignment"] = value;
                self:FrameLayoutRestoreAll();
            end,
        }
    end
    return options;
end

function Castbars:GetOptionsTable()
    local options =
    {
        type = "group",
        args =
        {
            general =
            {
                type = "group",
                name = "General Settings",
                handler = self,
                childGroups = "tree",
                args =
                {
                    general =
                    {
                        type = "group",
                        name = "General Settings",
                        guiInline = true,
                        order = 0,
                        args =
                        {
                            toggleconfigmode =
                            {
                                name = "Configuration Mode",
                                type = "toggle",
                                order = 1,
                                desc = "Toggle configuration mode to allow moving bars and setting appearance options",
                                get = function() return self.ConfigMode end,
                                set = function() self:Toggle() end,
                            },
                            merge =
                            {
                                name = "Merge Player/Vehicle",
                                type = "toggle",
                                order = 2,
                                desc = "Merge the Vehicle Castbar with the Player Castbar",
                                get = function() return self.db.profile.PetCastingBarFrame.Merge end,
                                set = function()
                                    self.db.profile.PetCastingBarFrame["Merge"] = not self.db.profile.PetCastingBarFrame["Merge"];
                                    if (self.db.profile.PetCastingBarFrame["Merge"]) then
                                        PetCastingBarFrame.configName = "CastingBarFrame";
                                        if (self.ConfigMode) then
                                            PetCastingBarFrame:Hide();
                                        end
                                        self.options.args.general.args.vehicle.hidden = true;
                                        self.options.args.general.args.vehicle.disabled = true;
                                    else
                                        PetCastingBarFrame.configName = "PetCastingBarFrame";
                                        if (self.ConfigMode) then
                                            PetCastingBarFrame:Show();
                                        end
                                        self.options.args.general.args.vehicle.hidden = false;
                                        self.options.args.general.args.vehicle.disabled = false;
                                    end
                                    self:FrameLayoutRestoreAll();
                                end,
                            },
                        },
                    },
                    player = self:GetOptionsTableForBar("CastingBarFrame", "Player Castbar", 1, true, false, true, true, false),
                    vehicle = self:GetOptionsTableForBar("PetCastingBarFrame", "Vehicle Castbar", 1, true, false, true, true, self.db.profile.PetCastingBarFrame["Merge"]),
                    target = self:GetOptionsTableForBar("TargetCastingBarFrame", "Target Castbar", 2, true, true, true, true, false),
                    mirror = self:GetOptionsTableForBar("MirrorTimer", "Mirror Timer Bars", 3),
                },
            },
            profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
        },
    };
    return options;
end

function Castbars:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("CastbarsDB",
    {
        profile =
        {
            ['**'] =
            {
                Width = 222,
                Height = 13,
                Texture = "Blizzard",
                Border = "Blizzard Tooltip",
                Font = "Friz Quadrata TT",
                FontSize = 12,
                FontOutline = true,
                TextAlignment = "CENTER",
                BorderBrightness = 0.2,
                Position = nil,
            },
            CastingBarFrame =
            {
                Show = true,
                ShowIcon = true,
                Position = {point = "BOTTOM", relpoint = "BOTTOM", x = 0, y = 200},
            },
            PetCastingBarFrame =
            {
                Show = true,
                ShowIcon = true,
                Merge = true,
            },
            TargetCastingBarFrame =
            {
                Show = true,
                ShowIcon = true,
                ShowShield = true,
                Border = "None",
                Position = {point = "CENTER", relpoint = "CENTER", x = 0, y = 200},
            },
            MirrorTimer =
            {
            },
        }
    });
    self.db.RegisterCallback(self, "OnProfileChanged", "FrameLayoutRestoreAll");
    self.db.RegisterCallback(self, "OnProfileCopied", "FrameLayoutRestoreAll");
    self.db.RegisterCallback(self, "OnProfileReset", "FrameLayoutRestoreAll");

    self.options = self:GetOptionsTable();
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Castbars", self.options);
    self.BlizzardOptionsFrameGeneral = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Castbars", nil, nil, "general");
    self.BlizzardOptionsFrameProfile = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Castbars", "Profiles", "Castbars", "profile");

    -- Prevent the UIParent from moving the CastingBarFrame around
    UIPARENT_MANAGED_FRAME_POSITIONS["CastingBarFrame"] = nil;

    -- Create target casting bar
    CreateFrame("StatusBar", "TargetCastingBarFrame", UIParent, "CastingBarFrameTemplate");
    TargetCastingBarFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
    CastingBarFrame_OnLoad(TargetCastingBarFrame, "target", false, true);

    -- Setup table with all frames
    self.frames = {CastingBarFrame, PetCastingBarFrame, TargetCastingBarFrame, MirrorTimer1, MirrorTimer2, MirrorTimer3};

    -- Customize the bars
    self:FrameCustomizeAll();

    -- Restore layout of all frames
    if (IsLoggedIn()) then
        self:FrameLayoutRestoreAll();
    end

    -- Listen to LibSharedMedia-3.0 callbacks
    self.SharedMedia.RegisterCallback(self, "LibSharedMedia_Registered", "FrameMediaRestoreAll")
    self.SharedMedia.RegisterCallback(self, "LibSharedMedia_SetGlobal", "FrameMediaRestoreAll")

    -- Hook OnUpdate
    local orgCastingBarFrame_OnUpdate = CastingBarFrame_OnUpdate;
    self.CastingBarFrame_OnUpdate = function(frame, elapsed, ...)
        orgCastingBarFrame_OnUpdate(frame, elapsed, ...);
        if (frame.timer and frame:IsVisible()) then
            if (frame.nextupdate < elapsed) then
                if (frame.casting) then
                    frame.timer:SetFormattedText("%.1f", max(frame.maxValue - frame.value, 0));
                elseif (frame.channeling) then
                    frame.timer:SetFormattedText("%.1f", max(frame.value, 0));
                elseif (self.ConfigMode) then
                    frame.timer:SetText("0.0");
                else
                    frame.timer:SetText();
                end;
                frame.nextupdate = 0.1;
            else
                frame.nextupdate = frame.nextupdate - elapsed;
            end
        end
    end
    CastingBarFrame_OnUpdate = self.CastingBarFrame_OnUpdate;

    -- Hook OnEvent
    local orgCastingBarFrame_OnEvent = CastingBarFrame_OnEvent;
    self.CastingBarFrame_OnEvent = function(frame, event, ...)
        if (event == "PLAYER_ENTERING_WORLD") then
            self:FrameLayoutRestoreAll();
        elseif (event == "PLAYER_TARGET_CHANGED") then
            event = "PLAYER_ENTERING_WORLD";
        end
        orgCastingBarFrame_OnEvent(frame, event, ...);
        if ((event == "UNIT_SPELLCAST_START") or (event == "UNIT_SPELLCAST_CHANNEL_START") or (event == "PLAYER_ENTERING_WORLD")) then
            if (frame:IsVisible()) then
                if (frame.latency) then
                    local min, max = frame:GetMinMaxValues();
                    local down, up, lag = GetNetStats();
                    local latency = (lag / 1000) / (max - min);
                    if (latency < 0) then latency = 0 elseif (latency > 1) then latency = 1 end
                    if (frame.channeling) then
                        frame.latency:ClearAllPoints();
                        frame.latency:SetPoint("LEFT", frame, "LEFT", 0, 0);
                    else
                        frame.latency:ClearAllPoints();
                        frame.latency:SetPoint("RIGHT", frame, "RIGHT", 0, 0);
                    end
                    frame.latency:SetWidth(frame:GetWidth() * latency);
                    frame.latency:Show();
                end
                self:FrameRestoreOnCast(frame);
            end
        elseif (event == "UNIT_SPELLCAST_STOP") then
            if (frame.latency and not frame.casting) then
                frame.latency:Hide();
            end
        elseif (event == "UNIT_SPELLCAST_CHANNEL_STOP") then
            if (frame.latency and not frame.channeling) then
                frame.latency:Hide();
            end
        end
    end
    CastingBarFrame_OnEvent = self.CastingBarFrame_OnEvent;

    self.GetOptionsTableForBar = nil;
    self.GetOptionsTable = nil;
    self.FrameCustomize = nil;
    self.FrameCustomizeAll = nil;
    self.OnInitialize = nil;
end

--[[ ConfigMode Support ]]--

function Castbars:ConditionCastbarForConfig(frame, text, show)
    frame:EnableMouse(true);
    frame.text:SetText(text)
    frame:SetStatusBarColor(0.0, 1.0, 0.0);
    frame:SetAlpha(1)
    frame.spark:Hide();
    frame.fadeOut = nil;
    if (show) then
        frame:Show();
    end
end

function Castbars:ConditionMirrorForConfig(frame, text, show)
    frame:EnableMouse(true);
    frame.paused = 1;
    frame.text:SetText(text);
    frame.statusBar:SetStatusBarColor(0.0, 1.0, 0.0);
    frame.statusBar:SetValue(select(2, frame.statusBar:GetMinMaxValues()));
    if (show) then
        frame:Show();
    end
end

Castbars.MirrorTimerFrame_OnEvent = MirrorTimerFrame_OnEvent;
Castbars.MirrorTimer_Show = MirrorTimer_Show;
Castbars.DoNothing = function() end;

function Castbars:Show()
    self.ConfigMode = true;
    CastingBarFrame_OnEvent = self.DoNothing;
    MirrorTimerFrame_OnEvent = self.DoNothing
    MirrorTimer_Show = self.DoNothing;

    self:ConditionCastbarForConfig(CastingBarFrame, "Player Castbar", true);
    self:ConditionCastbarForConfig(PetCastingBarFrame, "Vehicle Castbar", not self.db.profile.PetCastingBarFrame["Merge"]);
    self:ConditionCastbarForConfig(TargetCastingBarFrame, "Target Castbar", true);
    self:ConditionMirrorForConfig(MirrorTimer1, "Mirror Timer 1", true)
    self:ConditionMirrorForConfig(MirrorTimer2, "Mirror Timer 2", true)
    self:ConditionMirrorForConfig(MirrorTimer3, "Mirror Timer 3", true)
end

function Castbars:Hide()
    for i, frame in pairs(self.frames) do
        frame:EnableMouse(false);
        frame:Hide();
    end

    CastingBarFrame_OnEvent = self.CastingBarFrame_OnEvent;
    MirrorTimerFrame_OnEvent = self.MirrorTimerFrame_OnEvent;
    MirrorTimer_Show = self.MirrorTimer_Show;
    self.ConfigMode = false;
end

function Castbars:Toggle()
    if (self.ConfigMode) then
        self:Hide();
    else
        self:Show();
    end
end

CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {};
CONFIGMODE_CALLBACKS["Castbars"] = function(action)
    if (action == "ON") then Castbars:Show()
    elseif (action == "OFF") then Castbars:Hide() end
end
